@@ -452,7 +452,23 @@ options(help_type = "html")
 
 .rs.addJsonRpcHandler("show_vignette", function(topic, package)
 {
-   print(utils::vignette(topic, package))
+   # First, check for an explicitly registered vignette
+   vignette <- tryCatch(utils::vignette(topic, package), condition = identity)
+   if (!inherits(vignette, "condition"))
+      return(print(vignette))
+   
+   # Try falling back to opening bundled documentation that's not
+   # explicitly registered as a vignette.
+   exts <- c("pdf", "html")
+   for (ext in exts) {
+      suffix <- sprintf("doc/%s.%s", topic, ext)
+      path <- system.file(suffix, package = package, mustWork = FALSE)
+      if (nzchar(path))
+         return(browseURL(path))
+   }
+ 
+   # If we couldn't find the vignette, re-throw the original error.
+   stop(conditionMessage(vignette), call. = FALSE)
 })
 
 .rs.addFunction("getHelpColumn", function(name, src, envir = parent.frame())
@@ -469,52 +485,10 @@ options(help_type = "html")
    if (is.null(data))
       return(NULL)
    
-   column <- data[[name]]
-   
-   canUsePillar <- FALSE
-   if ("pillar" %in% loadedNamespaces())
-   {
-      pillar <- asNamespace("pillar")
-      canUsePillar <-
-         is.function(pillar$format_glimpse) &&
-         is.function(pillar$str_trunc) &&
-         is.function(pillar$get_pillar_type)
-   }
-   
-   if (canUsePillar)
-   {
-      formatted <- pillar$format_glimpse(column)
-      
-      bits <- c()
-      nchars <- 55
-      i <- 1
-      while (nchars > 1 && i <= length(column)) {
-         current <- formatted[i]
-         currentNChars <- nchar(current, keepNA = FALSE)
-         if (currentNChars > nchars) {
-            current <- pillar$str_trunc(current, nchars)
-         }
-         bits <- c(bits, current)
-         nchars <- nchars - currentNChars - 2
-         i <- i + 1
-      }
-      
-      if (length(bits) < length(column))
-      {
-         bits <- c(bits, paste0("<i>", pillar:::get_ellipsis(), "</i>"))
-      }
-      
-      description <- paste("<ul>", paste(paste0("<li>", bits, "</li>"), collapse = " "), "</ul>")
-      type <- pillar$get_pillar_type(column)
-      size <- length(formatted)
-   }
-   else 
-   {
-      described <- .rs.describeObject(data, name)
-      description <- described$description
-      type <- described$type
-      size <- described$length
-   }
+   described <- .rs.describeObject(data, name)
+   description <- described$description
+   type <- described$type
+   size <- described$length
    
    list(
       html = paste0("<h2></h2><h3>Description</h3><p>", description, "</p>"),
@@ -534,6 +508,10 @@ options(help_type = "html")
    showDataPreview <- getOption("rstudio.help.showDataPreview", default = TRUE)
    if (!showDataPreview)
       return(out)
+   
+   showDataPreview <- .rs.readUserPref("show_data_preview")
+   if (!identical(showDataPreview, TRUE))
+      return(out)
 
    # try and figure out the data + title
    data <- NULL
@@ -546,16 +524,21 @@ options(help_type = "html")
       data <- tryCatch(get(name, pos = pos), error = function(e) NULL)
    }
 
+   dataFoundAnywhere <- FALSE
    # if that failed, try to look it up anywhere
    if (is.null(data))
    {
       title <- paste0(src, "$", name)
       data <- .rs.getAnywhere(title, envir)
-   }
 
-   # if we still couldn't find any data, just use the help document
-   if (is.null(data))
-      return(out)
+      # if we still couldn't find any data, just use the help document
+      if (is.null(data))
+      {
+         return(out)
+      }
+   
+      dataFoundAnywhere <- TRUE
+   }
 
    # Generate the help pre-amble
    if (is.null(out))
@@ -592,12 +575,17 @@ options(help_type = "html")
 
    # build a uri
    attrs <- c(
-      env       = src,
-      obj       = name,
+      obj       = title,
       max_rows  = 1000,
       max_cols  = maxDisplayColumns
    )
    
+   # only include env if we didn't find the data from "anywhere"
+   if (!dataFoundAnywhere)
+   {
+      attrs <- c(attrs, env = src)
+   }
+   
    uri <- paste(
       "grid_resource/gridviewer.html",
       paste(names(attrs), attrs, sep = "=", collapse = "&"),
@@ -700,9 +688,18 @@ options(help_type = "html")
                                          package = NULL,
                                          help_type = "html")
 {
-   substitute(utils::help(TOPIC, package = PACKAGE, help_type = "html"),
-              list(TOPIC = topic,
-                   PACKAGE = package))
+   # build help call
+   call <- substitute(
+      utils::help(TOPIC, package = PACKAGE, help_type = "html"),
+      list(TOPIC = topic, PACKAGE = package)
+   )
+   
+   # support devtools shims
+   if ("devtools_shims" %in% search())
+      call[[1L]] <- quote(help)
+   
+   # return generated call
+   call
 })
 
 .rs.addFunction("packageHelpEncoding", function(packagePath)
@@ -769,7 +766,7 @@ options(help_type = "html")
          
          expr = {
             call <- .rs.makeHelpCall(topic, package)
-            eval(call)
+            eval(call, envir = globalenv())
          },
          
          error = function(e) {
@@ -781,13 +778,29 @@ options(help_type = "html")
    if (length(helpfiles) <= 0)
       return()
    
-   file <- helpfiles[[1]]
-   path <- dirname(file)
-   dirpath <- dirname(path)
-   pkgname <- basename(dirpath)
-   
-   query <- paste("/library/", pkgname, "/html/", basename(file), ".html", sep = "")
-   html <- suppressWarnings(tools:::httpd(query, NULL, NULL))$payload
+   # support devtools help
+   if (inherits(helpfiles, "dev_topic") && "pkgload" %in% loadedNamespaces())
+   {
+      .rs.tryCatch({
+         pkgname <- helpfiles$pkg
+         docPath <- file.path(tempdir(), sprintf(".R/doc/html/%s.html", helpfiles$topic))
+         dir.create(dirname(docPath), recursive = TRUE, showWarnings = FALSE)
+         pkgload:::topic_write_html(helpfiles, path = docPath)
+      })
+      
+      html <- paste(readLines(docPath, warn = FALSE), collapse = "\n")
+   }
+   else
+   {
+      # regular old help
+      file <- helpfiles[[1]]
+      path <- dirname(file)
+      dirpath <- dirname(path)
+      pkgname <- basename(dirpath)
+      
+      query <- paste("/library/", pkgname, "/html/", basename(file), ".html", sep = "")
+      html <- suppressWarnings(tools:::httpd(query, NULL, NULL))$payload
+   }
    
    # resolve associated package from helpfile path -- note that help file
    # paths have the format:
